--[[

Function for loading and saving settings and data files.

]]--

maxServerID = 0;

-- load window size/position settings from Character-level file
function _G.LoadSettings()
	_G.settings = DecodeNumbers(Turbine.PluginData.Load(Turbine.DataScope.Character, "KillTracker"));
	
	local upToDate = true;

   -- check that settings file is valid and not for an old version of the plugin
	if ((type(settings) ~= "table") or (settings.versionNo == nil) or (string.gsub(settings.versionNo, "%.", "") < string.gsub(versionNo, "%.", ""))) then
		upToDate = false;
		
      if (type(settings) ~= "table") then
			_G.settings = {};
		end
		
      -- if the settings are too out-dated, put in new defaults
		if ((settings.versionNo == nil) or (string.gsub(settings.versionNo, "%.", "") < "091")) then
         _G.settings = {};
			settings.leftPos = (Turbine.UI.Display:GetWidth() - minWidth) / 2;
         settings.topPos = (Turbine.UI.Display:GetHeight() - minHeight) / 2;
         settings.width = minWidth;
         settings.height = minHeight;
			settings.iconLeft = 0;
         settings.iconTop = Turbine.UI.Display:GetHeight() - 32;
		end
		
      -- update version number to current
		settings.versionNo = versionNo;
	end

   -- ensure saved settings for width and height are at least at the set minimums
   if (settings.width < minWidth) then
      upToDate = false;
      
      settings.width = minWidth;
   end

   if (settings.height < minHeight) then
      upToDate = false;
      
      settings.height = minHeight;
	end

	-- account for changed resolution
	if (settings.width > Turbine.UI.Display.GetWidth()) then
		settings.width = Turbine.UI.Display.GetWidth();
	end

	if (settings.leftPos < 0) then
		settings.leftPos = 0;
	end
	
	if (settings.leftPos > Turbine.UI.Display.GetWidth() - settings.width) then
		settings.leftPos = Turbine.UI.Display.GetWidth() - settings.width;
	end

	if (settings.height > Turbine.UI.Display.GetHeight()) then
		settings.height = Turbine.UI.Display.GetHeight();
	end

	if (settings.topPos < 0) then
		settings.topPos = 0;
	end

	if (settings.topPos > Turbine.UI.Display.GetHeight() - settings.height) then
		settings.topPos = Turbine.UI.Display.GetHeight() - settings.height;
	end

	if (settings.iconLeft < 0) then
		settings.iconLeft = 0;
	end
	
	if (settings.iconLeft > Turbine.UI.Display.GetWidth() - 32) then
		settings.iconLeft = Turbine.UI.Display.GetWidth() - 32;
	end

	if (settings.iconTop < 0) then
		settings.iconTop = 0;
	end

	if (settings.iconTop > Turbine.UI.Display.GetHeight() - 32) then
		settings.iconTop = Turbine.UI.Display.GetHeight() - 32;
	end
   
	-- save default settings if applied
   if (not upToDate) then SaveSettings() end
end

-- save window size/position settings to Character-level file
function _G.SaveSettings()
	Turbine.PluginData.Save(Turbine.DataScope.Character, "KillTracker", EncodeNumbers(settings));
end

-- load main kill log data and ServerID counter from Account-level file
function _G.LoadData()
	local tmpData = DecodeNumbers(Turbine.PluginData.Load(Turbine.DataScope.Account, "KillTracker"));
   
   if (tmpData ~= nil) then
      -- data file corrupt, disable saving so it can be fixed manually, hopefully without data loss
      if (type(tmpData) ~= "table") then
         saveData = false;

         LogMessage("KillTracker: "..L.LoadFailure, 0)

         _G.killsTable = {};
      else
         maxServerID = tmpData[1];
         _G.killsTable = tmpData[2];
      end
   -- data file not found, set defaults
   else
      LogMessage("KillTracker: Data file not found.", 1);

      _G.killsTable = {};
   end
end

-- save main kill log data and ServerID counter to Account-level file
function _G.SaveData()
   -- don't save if data file load failed
   if (saveData) then
      local tmpData = {};

      tmpData[1] = maxServerID;
      tmpData[2] = killsTable;

      Turbine.PluginData.Save(Turbine.DataScope.Account, "KillTracker", EncodeNumbers(tmpData));
   end
end

-- load/set/save Server ID for current server from/to Server-level file
function _G.LoadServerID()
	local serverID = Turbine.PluginData.Load(Turbine.DataScope.Server, "KillTracker");

	-- no Server ID set, set one
   if (serverID == nil) then
      -- don't save if data file load failed
      if (saveData) then
         -- increment max Server ID from Account-level file
         maxServerID = maxServerID + 1;

         -- set new Server ID, as string
         serverID = ""..maxServerID;
         
         -- write new max Server ID to Account-level file first to ensure no re-use
         SaveData();
      
         -- write new Server ID to Server-level file
         Turbine.PluginData.Save(Turbine.DataScope.Server, "KillTracker", serverID);
      -- data file load failed, system in view-only mode, Server ID doesn't matter
      else
         serverID = "999";
      end      
	end
   
   -- set character unique name as CharacterName(ServerID), used to distinguish
   --   same-named characters on different servers within same account
   player.uniqueName = player.name.."("..serverID..")";

   -- Once this file is written it could be manually edited to use any
   -- unique string, such as the actual server name, instead of the number. 
   -- Just be careful - if character kill data has already been saved for any
   -- character on this server, the Account-level data file will also need to
   -- be adjusted so the unique character names match the new ServerID value:
   --    ["<num>7.000000</num>"] = "CharacterName(ServerID)",
   -- This currently isn't displayed anywhere, but future functionality is
   -- planned that would allow cross-character comparisons, where this unique
   -- name would be used for display. 
end
