--[[

Miscellaneous functions, including:

- simple math.round function
- a locale-aware number formatting function
- callback handling functions
- chat message output function
- window size validator
- plugin version number extractor
- encode/decode functions for data saving/loading

]]--

-- round a number
function _G.RoundNum(num, idp)
  local mult = 10 ^ (idp or 0)
  
  return math.floor(num * mult + 0.5) / mult
end

-- format a whole number
function _G.FormatNumber(number)
   local result = "";

   local wholenumber = string.match(tostring(number), "^[%+%-]?(%d*)%.?.*$");
   
   while (string.len(wholenumber) > 3) do
      result = L.NumberSplitChar..string.sub(wholenumber, -3, -1)..result;

      wholenumber = string.sub(wholenumber, 1, -4); -- remove last 3 digits
   end
   
   return wholenumber..result;
end

-- for compatability with other plugins, this method is used to not override the same events
function _G.AddCallback(object, event, callback)
	if (object[event] == nil) then
		object[event] = callback;
	else
		if (type(object[event]) == "table") then
			table.insert(object[event], callback);
		else
			object[event] = {object[event], callback};
		end
	end
end

function _G.RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);

            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);

                    break;
                end
            end
        end
    end
end

-- ensure a window remains above a minimum size, and is in the display
function _G.ValidateWindowSize(x, y, w, h, minW, minH, anchorBottom, anchorRight, topExtra, rightExtra, bottomExtra, leftExtra)
	if (topExtra == nil) then topExtra = 0 end
	if (rightExtra == nil) then rightExtra = topExtra end
	if (bottomExtra == nil) then bottomExtra = rightExtra end
	if (leftExtra == nil) then leftExtra = bottomExtra end

	xShift = math.max(-x - leftExtra, 0) + math.max(x + w - rightExtra - Turbine.UI.Display:GetWidth(), 0);
	yShift = math.max(-y - topExtra, 0) + math.max(y + h - bottomExtra - Turbine.UI.Display:GetHeight(), 0);
	
	wShift = math.max(0, minW - w);
	hShift = math.max(0, minH - h);
	
	if (anchorBottom == nil and anchorRight == nil) then
		x = x - RoundNum(wShift / 2) + math.max(-x - leftExtra, 0);
		y = y - RoundNum(hShift / 2) + math.max(-y - topExtra, 0);
	end
	
	x = x - (anchorRight and wShift or 0) + (anchorRight and xShift or 0);
	y = y - (anchorBottom and hShift or 0) + (anchorBottom and yShift or 0);
	w = math.max(w, minW) - xShift;
   h = math.max(h, minH) - yShift;
	
   return x, y, w, h;
end

-- write a formatted message to the chat window
function _G.LogMessage(message, minDebugLvl)
   if (debugLevel >= minDebugLvl) then
      if (logHightlight) then
         Turbine.Shell.WriteLine("<rgb=#FFFF00>"..message.."</rgb>");
      else
         Turbine.Shell.WriteLine(message);
      end
   end
end

-- automatically extract plugin version number
function _G.GetVersionNo()
	for _, plugin in ipairs(Turbine.PluginManager.GetAvailablePlugins()) do
		if (plugin.Name == "KillTracker") then
			return plugin.Version;
		end
	end

	return "";
end

-- global function to encode numbers into a save-proof format (in place)
function _G.EncodeNumbers(object)
	if (type(object) == "number") then
		return ("<num>%f</num>"):format(object);
	elseif (type(object) ~= "table") then
		return object;
	else
		local newTable = {};
		for index, value in pairs(object) do
			newTable[EncodeNumbers(index)] = EncodeNumbers(value);
		end
		return newTable;
	end
end

-- global function to decode (convert from strings) stored numbers
function _G.DecodeNumbers(object)
	-- parse strings into numbers where applicable
	if (type(object) == "string") then
		local number,separator,decimal = object:match("<num>(-?%d+)([%.,])(%d+)</num>");
		if (number ~= nil) then
			local decodedNumber = tonumber(number..separator..decimal);
			if (decodedNumber ~= nil) then return decodedNumber end
			return tonumber(number..(separator == "," and "." or ",")..decimal);
		else
			return object;
		end
	-- return any other data types
	elseif (type(object) ~= "table") then
		return object;
	-- decode each element inside a table and return
	else
		local newTable = {}
		for index, value in pairs(object) do
			newTable[DecodeNumbers(index)] = DecodeNumbers(value);
		end
		return newTable;
	end
end
